----------------------------------------------------------------------------------
--    This file is part of OsciOne: an open source Android oscilloscope.
--
--    OsciOne is free software: you can redistribute it and/or modify
--    it under the terms of the GNU General Public License as published by
--    the Free Software Foundation, either version 3 of the License, or
--    (at your option) any later version.
--
--    OsciOne is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--    GNU General Public License for more details.
--
--    You should have received a copy of the GNU General Public License
--    along with OsciOne.  If not, see <http://www.gnu.org/licenses/>.
--
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
USE ieee.numeric_std.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity controller is
    Port ( data : in  STD_LOGIC_VECTOR (7 downto 0);
           newData : in STD_LOGIC;
           reset : in  STD_LOGIC;
           clk : in  STD_LOGIC;
           gainA : out  STD_LOGIC_VECTOR (3 downto 0);
           gainB : out  STD_LOGIC_VECTOR (3 downto 0);
			  interLeave : out std_logic;
			  trigMode : out STD_LOGIC;
			  trigPolarity : out STD_LOGIC;
			  trigChannel : out STD_LOGIC;
			  trigLevel	: out unsigned(5 DOWNTO 0);
			  FIFOADDR0 : out STD_LOGIC;
			  FIFOADDR1 : out STD_LOGIC
			  );
end controller;

architecture behav of controller is

begin

clocked: PROCESS(clk, reset)
variable command : std_logic_vector(1 DOWNTO 0) := "00";
BEGIN
IF reset='0' THEN
	trigMode <= '0';
	gainA <= "0001";
ELSIF rising_edge(clk) THEN

	IF newData='1' THEN
		command(1 DOWNTO 0) := data(7 DOWNTO 6);

		CASE command IS
			-- gain
			WHEN "00" =>
				CASE data(2 DOWNTO 0) IS
				WHEN "000" => 	gainA <= "0001";
				WHEN "001" => 	gainA <= "0010";
				WHEN "010" => 	gainA <= "0100";
				WHEN "011" => 	gainA <= "1000";
				WHEN OTHERS => gainA <= "0000";
				END CASE;
			
				CASE data(5 DOWNTO 3) IS
				WHEN "000" => 	gainB <= "0001";
				WHEN "001" => 	gainB <= "0010";
				WHEN "010" => 	gainB <= "0100";
				WHEN "011" => 	gainB <= "1000";
				WHEN OTHERS => gainB <= "0000";
				END CASE;
			
			-- mode
			WHEN "01" =>			
			trigMode <= data(0);
			trigChannel <= data(1);
			trigPolarity <= data(2);
			
			CASE data(0) IS
			WHEN '0' => 	FIFOADDR0 <= '0'; -- continuous
								FIFOADDR1 <= '0';
			WHEN OTHERS =>	FIFOADDR0 <= '0'; -- tiggered
								FIFOADDR1 <= '1';
			END CASE;
			
			-- trigger Level
			WHEN "10" =>
			trigLevel <= unsigned(data(5 downto 0));
			
			-- SLWR interleave
			WHEN "11" =>
			interLeave <= data(0);
			
			WHEN OTHERS =>
		END CASE;

		
	END IF;
	
	--gainA <= "0001";
	--gainB <= "0001";
	
END IF;
END PROCESS clocked;

end behav;

