----------------------------------------------------------------------------------
--    This file is part of OsciOne: an open source Android oscilloscope.
--
--    OsciOne is free software: you can redistribute it and/or modify
--    it under the terms of the GNU General Public License as published by
--    the Free Software Foundation, either version 3 of the License, or
--    (at your option) any later version.
--
--    OsciOne is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--    GNU General Public License for more details.
--
--    You should have received a copy of the GNU General Public License
--    along with OsciOne.  If not, see <http://www.gnu.org/licenses/>.
--
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
--use IEEE.STD_LOGIC_ARITH.ALL;
--use IEEE.STD_LOGIC_UNSIGNED.ALL;
USE ieee.numeric_std.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity oscione_top is
    Port ( clk : in  STD_LOGIC;
           reset : in  STD_LOGIC;
           SLWR : out  STD_LOGIC;
           gainA : out  STD_LOGIC_VECTOR (3 downto 0);
           gainB : out  STD_LOGIC_VECTOR (3 downto 0);
           adcA : in  unsigned (7 downto 0);
           adcB : in  unsigned (7 downto 0);
			  
			  SLRD : out std_logic;
			  FIFOADDR0 : out std_logic;
			  FIFOADDR1 : out std_logic;
			  SLOE : out std_logic;
			  PKTEND : out std_logic;
			  
           SDA : inout  STD_LOGIC;
           SCL : in  STD_LOGIC;
           FIFO : out  STD_LOGIC_VECTOR (15 downto 0);
			  ACLK : out STD_LOGIC;
			  BCLK : out STD_LOGIC;
			  
			  probeTuning : out STD_LOGIC
			  );
end oscione_top;

architecture Structural of oscione_top is
-- component declaration
COMPONENT controller
Port ( 	  data : in  STD_LOGIC_VECTOR (7 downto 0);
           newData : in STD_LOGIC;
           reset : in  STD_LOGIC;
           clk : in  STD_LOGIC;
           gainA : out  STD_LOGIC_VECTOR (3 downto 0);
           gainB : out  STD_LOGIC_VECTOR (3 downto 0);
			  interLeave : out std_logic;
			  trigMode : out STD_LOGIC;
			  trigPolarity : out STD_LOGIC;
			  trigChannel : out STD_LOGIC;
			  trigLevel	: out unsigned(5 DOWNTO 0);
			  FIFOADDR0 : out STD_LOGIC;
			  FIFOADDR1 : out STD_LOGIC
			  );
END COMPONENT;

COMPONENT i2c_slave
Port ( 	  pSCL : in  STD_LOGIC;
           pSDA : inout  STD_LOGIC;
           clk : in  STD_LOGIC;
		     reset : in STD_LOGIC;
           newDATA : out  STD_LOGIC := '0';
           DATA : out  STD_LOGIC_VECTOR (7 downto 0) := "00000000"
);
END COMPONENT;

COMPONENT sampling
Port ( 	  clk : in STD_LOGIC;
			  reset : in STD_LOGIC;
			  interleave : in std_logic;
           trigMode : in  STD_LOGIC;
           trigPolarity : in  STD_LOGIC;
           trigChannel : in  STD_LOGIC;
           trigLevel : in  unsigned (5 downto 0);
           adcA : in  unsigned (7 downto 0);
           adcB : in  unsigned (7 downto 0);
			  FIFO : out STD_LOGIC_VECTOR (15 downto 0);
			  SLWR : out STD_LOGIC;
			  probeTuning : out STD_LOGIC;
			  PKTEND : out STD_LOGIC
			  );
END COMPONENT;

-- embedded configuration

-- signal declaration
signal interLeave : STD_LOGIC;
signal newDATA : STD_LOGIC;
signal DATA : std_logic_vector(7 DOWNTO 0);

signal trigMode : std_logic;
signal trigPolarity : std_logic;
signal trigChannel : std_logic;
signal trigLevel : unsigned(5 DOWNTO 0);


begin
inst1 : controller PORT MAP(
						data => data,
						newData => newData,
						reset => reset,
						clk => clk,
						gainA => gainA,
						gainB => gainB,
						interLeave => interLeave,
						trigMode => trigMode,
						trigPolarity => trigPolarity,
						trigChannel => trigChannel,
						trigLevel => trigLevel,
						FIFOADDR0 => FIFOADDR0,
						FIFOADDR1 => FIFOADDR1
						);
						
						
inst2 : i2c_slave PORT MAP(
						pSCL => SCL,
						pSDA => SDA,
						clk => clk,
						reset => reset,
						newDATA => newDATA,
						DATA => DATA);
						
inst3 : sampling PORT MAP(
						clk => clk,
						reset => reset,
						interLeave => interLeave,
						trigMode => trigMode,
						trigPolarity => trigPolarity,
						trigChannel => trigChannel,
						trigLevel => trigLevel,
						adcA => adcA,
						adcB => adcB,
						FIFO => FIFO,
						SLWR => SLWR,
						probeTuning => probeTuning,
						PKTEND => PKTEND
						);

ACLK <= clk;
BCLK <= clk;
SLRD <= '0';
SLOE  <= '0';
	  			
end Structural;
