----------------------------------------------------------------------------------
--    This file is part of OsciOne: an open source Android oscilloscope.
--
--    OsciOne is free software: you can redistribute it and/or modify
--    it under the terms of the GNU General Public License as published by
--    the Free Software Foundation, either version 3 of the License, or
--    (at your option) any later version.
--
--    OsciOne is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--    GNU General Public License for more details.
--
--    You should have received a copy of the GNU General Public License
--    along with OsciOne.  If not, see <http://www.gnu.org/licenses/>.
--
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
--use IEEE.STD_LOGIC_ARITH.ALL;
--use IEEE.STD_LOGIC_UNSIGNED.ALL;
USE ieee.numeric_std.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity sampling is
    Port ( clk : in STD_LOGIC;
			  reset : in STD_LOGIC;
			  interLeave : in  std_logic;
           trigMode : in  STD_LOGIC;
           trigPolarity : in  STD_LOGIC;
           trigChannel : in  STD_LOGIC;
           trigLevel : in  unsigned (5 downto 0);
           adcA : in  unsigned (7 downto 0);
           adcB : in  unsigned (7 downto 0);
			  FIFO : out STD_LOGIC_VECTOR (15 downto 0);
			  SLWR : out STD_LOGIC;
			  probeTuning : out STD_LOGIC;
			  PKTEND : out STD_LOGIC
			  );
end sampling;

architecture Behavioral of sampling is

signal adcA_int : unsigned(7 DOWNTO 0) := "00000000";
signal adcB_int : unsigned(7 DOWNTO 0) := "00000000";
signal oldSample : unsigned(7 DOWNTO 0) := "00000000";
signal counter : unsigned(12 DOWNTO 0) := (OTHERS => '0');

TYPE state_type IS (init,trigger,asserting,hold);
signal state : state_type := init;

begin
clocked: PROCESS(reset, clk)
BEGIN
IF reset='0' THEN
state <= init;
SLWR <= '1';
ELSIF rising_edge(clk) THEN

CASE trigMode IS
-- continuous
WHEN '0' =>

	CASE interLeave IS
	WHEN '0' =>
		IF counter(2 DOWNTO 0)="000" THEN
		SLWR <= '0';
		ELSE
		SLWR <= '1';
		END IF;
	WHEN OTHERS =>
		IF counter(9 DOWNTO 0)="0000000000" THEN
		SLWR <= '0';
		ELSE
		SLWR <= '1';
		END IF;
	END CASE;
	counter <= counter + 1;
	state <= init;

-- triggered
WHEN OTHERS =>

CASE state IS
WHEN init => 	
					CASE trigChannel IS
					-- channel A
					WHEN '0' =>		oldSample <= adcA_int;
					-- channel B
					WHEN OTHERS =>	oldSample <= adcB_int;
					END CASE;
					
					counter <= (OTHERS => '0');
					PKTEND <= '1';
					
					state <= trigger;
					

WHEN trigger =>
	PKTEND <= '0';

	CASE trigChannel IS
	-- channel A
	WHEN '0' => CASE trigPolarity IS
					-- rising edge
					WHEN '0' => IF adcA_int > (trigLevel&"00") AND oldSample <= (trigLevel&"00") THEN
									state <= asserting;
									END IF;
					-- falling egde
					WHEN OTHERS => IF adcA_int < (trigLevel&"00") AND oldSample >= (trigLevel&"00") THEN
									state <= asserting;
									END IF;
					END CASE;
					oldSample <= adcA_int;
	
	-- channel B
	WHEN OTHERS =>	CASE trigPolarity IS
					-- rising edge
					WHEN '0' => IF adcB_int > (trigLevel&"00") AND oldSample <= (trigLevel&"00") THEN
									state <= asserting;
									END IF;
					
					-- falling edge
					WHEN OTHERS =>	IF adcB_int < (trigLevel&"00") AND oldSample >= (trigLevel&"00") THEN
									state <= asserting;
									END IF;
					END CASE;
					oldSample <= adcB_int;
	
	END CASE;
	
	
WHEN asserting =>
					counter <= counter + 1;
					IF counter < 1024 THEN
					SLWR <= '0';
					ELSE
					SLWR <= '1';
					state <= hold;
					END IF;
					
WHEN hold =>	counter <= counter + 1;
					
END CASE;


	
END CASE;

adcA_int <= NOT adcA;
adcB_int <= NOT adcB;

END IF;
END PROCESS;

FIFO <= std_logic_vector(adcA_int(7 downto 0)) & std_logic_vector(adcB_int(7 downto 0));
probeTuning <= '1' WHEN counter < 4095 ELSE
					'0';
						 

end Behavioral;

